/*Copyright 2009-2011 David McCallum

This file is part of EbonHack.

    EbonHack is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    EbonHack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with EbonHack.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "NGSettings.hpp"
#include "ImageLoader.hpp"

using namespace std;

QFont NGSettings::telnetFont;

const string NGSettings::DATA_PATH = "data/";

int NGSettings::fontOffset = 0;
int NGSettings::logLines = 0;

char NGSettings::upLeftKey = ' ';
char NGSettings::upKey = ' ';
char NGSettings::upRightKey = ' ';
char NGSettings::rightKey = ' ';
char NGSettings::downLeftKey = ' ';
char NGSettings::downKey = ' ';
char NGSettings::downRightKey = ' ';
char NGSettings::leftKey = ' ';
bool NGSettings::useNumpad = true;

NGSettings::NGSettings(void)
{
    string fontName = ConfigWriter::loadString("game_config.txt", "Font Name");

    //### Set up the telnet font ###
    telnetFont.setFamily(QString::fromStdString(fontName));
    fontOffset = ConfigWriter::loadInt("game_config.txt", "Font Offset 16");
    setSpriteSize(ImageLoader::DEFAULT_SPRITE_SIZE);

    //### Set up the keypad ###
    useNumpad = ConfigWriter::loadBool("game_config.txt", "Numpad Keys");
    setKeys();

    //### Load the number of lines in the history log ###
    logLines = ConfigWriter::loadInt("game_config.txt", "History Size");
}//constructor

NGSettings::~NGSettings(void)
{
}//destructor

void NGSettings::setSpriteSize(uint8_t spriteSize)
{
    int fontSize = 0;//the size of the font in points

    if (spriteSize >= 32)
	{
		fontOffset = ConfigWriter::loadInt("game_config.txt", "Font Offset Large");
        fontSize = ConfigWriter::loadInt("game_config.txt", "Font Size 32");
	}//if spriteSize

    else if (spriteSize >= 24)
	{
		fontOffset = ConfigWriter::loadInt("game_config.txt", "Font Offset Large");
        fontSize = ConfigWriter::loadInt("game_config.txt", "Font Size 24");
	}//else if spriteSize

    else if (spriteSize >= 20)
	{
		fontOffset = ConfigWriter::loadInt("game_config.txt", "Font Offset Large");
        fontSize = ConfigWriter::loadInt("game_config.txt", "Font Size 20");
	}//else if spriteSize

    else if (spriteSize >= 16)
	{
		fontOffset = ConfigWriter::loadInt("game_config.txt", "Font Offset 16");
        fontSize = ConfigWriter::loadInt("game_config.txt", "Font Size 16");
	}//else spriteSize

	else
    {
        fontOffset = ConfigWriter::loadInt("game_config.txt", "Font Offset 16");
        fontSize = ConfigWriter::loadInt("game_config.txt", "Font Size 12");
    }//else if spriteSize

    telnetFont.setPointSize(fontSize);
}//setSpriteSize

void NGSettings::setKeys(void)
{
    if (useNumpad)
    {
        upLeftKey = '7';
        upKey = '8';
        upRightKey = '9';
        rightKey = '6';
        downRightKey = '3';
        downKey = '2';
        downLeftKey = '1';
        leftKey = '4';
    }//if useNumpad
    else
    {
        upLeftKey = 'y';
        upKey = 'k';
        upRightKey = 'u';
        rightKey = 'l';
        downRightKey = 'n';
        downKey = 'j';
        downLeftKey = 'b';
        leftKey = 'h';
    }//else useNumpad
}//setKeys

QFont* NGSettings::getTelnetFont(void)
{
    return &telnetFont;
}//getTelnetFont

char NGSettings::getUpLeftKey(void)
{
    return upLeftKey;
}//getUpLeftKey

char NGSettings::getUpKey(void)
{
    return upKey;
}//getUpKey

char NGSettings::getUpRightKey(void)
{
    return upRightKey;
}//getUpRightKey

char NGSettings::getRightKey(void)
{
    return rightKey;
}//getRightKey

char NGSettings::getDownRightKey(void)
{
    return downRightKey;
}//getDownRightKey

char NGSettings::getDownKey(void)
{
    return downKey;
}//getDownKey

char NGSettings::getDownLeftKey(void)
{
    return downLeftKey;
}//getDownLeftKey

char NGSettings::getLeftKey(void)
{
    return leftKey;
}//getLeftKey

bool NGSettings::getUseNumpad(void)
{
    return useNumpad;
}//getUseNumpad

int NGSettings::getFontOffset(void)
{
    return fontOffset;
}//getFontOffset

int NGSettings::getLogLines(void)
{
    return logLines;
}//getLogLines
